︠338fe003-a52d-4873-9dea-d8279bdc68f0︠
"""This sheet is used to compute the matrix B from the paper Stable symmetric polynomials and the Schur-Agler class.  It would help to refer to equation (3.4) from the arxiv version of that paper."""
︡8f224ff2-5ea6-4e66-9f67-50750f2f443c︡{"stdout": "'This sheet is used to compute the matrix B from the paper Stable symmetric polynomials and the Schur-Agler class.  It would help to refer to equation (3.4) from the arxiv version of that paper.'"}︡
︠2990d54a-955d-4336-a3f1-0ce905eeb399︠
"""The matrix B has its rows and columns indexed by subsets of {0,1,2,...,d-2}.  I found it convenient to represent the subsets using binary expansions.  The following function just gives the number of 1's in the binary expansion of a natural number (which would correspond to the size of a subset)."""
︡02bbd34e-4155-4a51-9ae7-5d282a23c042︡{"stdout": "\"The matrix B has its rows and columns indexed by subsets of {0,1,2,...,d-2}.  I found it convenient to represent the subsets using binary expansions.  The following function just gives the number of 1's in the binary expansion of a natural number (which would correspond to the size of a subset).\""}︡
︠f4ff0e31-e56a-40c3-b0f7-f91ce78a3559︠
"""For example, the subset {0,2,3} of {0,1,..,4} would correspond to the bitstring 01101 = 1+2^2+2^4=21, and bitsize(21) would give 3."""
︡72a0447b-e470-4037-a7d0-a954f37a61f5︡{"stdout": "'For example, the subset {0,2,3} of {0,1,..,4} would correspond to the bitstring 01101 = 1+2^2+2^4=21, and bitsize(21) would give 3.'"}︡
︠059998b7-9d34-401d-a716-bf64c2ac9306︠
def bitsize(n):
    m = 0
    while n != 0:
        m = (n % 2) + m
        n = n >> 1
    return m
︡3d1ada60-155c-4f58-9612-b5c275f824a6︡︡
︠5701a62a-0cd4-4c7f-861c-d0aab2f4c6a8︠
bitsize(21)
︡e2e1da53-e06a-417a-9b1a-1562e3900599︡{"stdout": "3"}︡
︠9c583183-64fa-4743-a37e-dd75a5b2b0ce︠
"""This next function returns the location of the 1 furthest to the right in a binary expansion.  Again, 21 = 01101 and so findbit(21) should return 0. While findbit(16) should return 4 since 16=10000 in binary and 1 is in the 2^4 place."""
︡93e35675-8c8a-4b5c-9c6c-dedd9ccd881e︡{"stdout": "'This next function returns the location of the 1 furthest to the right in a binary expansion.  Again, 21 = 01101 and so findbit(21) should return 0. While findbit(16) should return 4 since 16=10000 in binary and 1 is in the 2^4 place.'"}︡
︠91c41015-5cb4-4143-b910-583991aa0d2c︠
def findbit(n):
    m = 0
    while n != 0:
        if n % 2 == 1:
            n = 0
        else:
            n = n >> 1
        m = m+1
    return m-1
︡5c0f8710-4e61-4173-929b-5c73aaff90a9︡︡
︠28f9653c-0001-4558-a3ca-85c5b8b13185︠
findbit(21), findbit(16)
︡de5d5d82-8a8b-4aec-8310-c0ef1139560b︡{"stdout": "(0, 4)"}︡
︠62a910f2-1a07-4446-a52c-6ad4c3c95252︠
"""This next function gives the matrix B from the paper Stable symmetric polynomials and the Schur-Agler class.  See equation (3.4) from the arxiv version (which may differ in older versions).  The input is a vector of coefficients of a polynomial and the output is the corresponding matrix B."""
︡153810af-4e4b-4e6e-95da-420f5706f3a5︡{"stdout": "'This next function gives the matrix B from the paper Stable symmetric polynomials and the Schur-Agler class.  See equation (3.4) from the arxiv version (which may differ in older versions).  The input is a vector of coefficients of a polynomial and the output is the corresponding matrix B.'"}︡
︠f3dd474c-a484-498a-a384-13b3baf9a95f︠
def Bmatrix(coefs):
    B = []                          #initialize the array 
    if len(coefs) != 0:             #Make sure there is at least 1 coefficient
        L = len(coefs)              #The number of coefficients of the polynomial
        N = 2^(L-2)                 #This will be the number of rows and columns of the matrix
        for i in range(N):          #Begin to construct the array
            B.append([])            #Put an empty row into the array
            i_len = bitsize(i)      #The number i corresponds to a subset with i_len elements  
            for j in range(N):      #Start looping over the columns  
                j_len = bitsize(j)  #The number j corresponds to a subset with j_len elements
                # 
                # The next expression computes part of the left hand side of equation (3.4) from 
                # the paper.
                #
                schurcohn = coefs[i_len]*coefs[j_len] - coefs[L-1- j_len]* coefs[L - 1 - i_len]
                # 
                # The next expression computes the full expression on the left hand side of equation (3.4) divided by
                # (d-j-k-1) which occurs on the right side.
                #
                C = (1/(L-1 - i_len - j_len + bitsize(i & j)))*(1/(binomial(L-1, i_len)*binomial(L-1,j_len)))*(schurcohn)
                if i & j == 0:   #If the subsets corresponding to i and j do not intersect the recursion in equation 
                                 # (3.4) stops.   
                    B[i].append(C)
                else:            #Otherwise, we need to compute B^{i-1}_{j-1,k-1} which we have already done. 
                    back_i = i - 2^(findbit(i&j))
                    back_j = j - 2^(findbit(i&j))
                    D = B[back_i][back_j]
                    B[i].append(C + bitsize(i & j) * D/(L-1 - i_len - j_len + bitsize(i & j)))
    return B
︡0e63a769-81f1-4d06-8886-05944f2fe73c︡︡
︠4af053fe-bc6f-4733-9ca5-14c66c78d22d︠
"""For example if my polynomial is p(z) = 1-z=1+(-1)z+0z^2+0z^3+0z^4 which I want to view as a polynomial of degree at most 4 (the degree is important for symmetrization) I could set:"""
︡815c5771-7ba1-43b2-ad62-c3bf6d95bd13︡{"stdout": "'For example if my polynomial is p(z) = 1-z=1+(-1)z+0z^2+0z^3+0z^4 which I want to view as a polynomial of degree at most 4 (the degree is important for symmetrization) I could set:'"}︡
︠f4c84752-8998-479e-bb73-dd5beb5aa4f1︠
p = [1,-1,0,0,0]
︡52154b18-612a-4c89-bd29-ccf6ba227e9e︡︡
︠462adc32-1b9a-4c33-94c7-de400a2b0436︠
"""Then, the following command would give me an array of the matrix B, which I need to convert into a matrix"""
︡228daea0-ab35-485f-8cec-1c9108068ce0︡{"stdout": "'Then, the following command would give me an array of the matrix B, which I need to convert into a matrix'"}︡
︠b924b822-3895-4a32-8fb1-46572e8cc5cd︠
X = matrix(Bmatrix(p))
︡cf0025cd-6480-4feb-84e9-eab1d5ea17e7︡︡
︠fdf04d98-fede-42a5-9666-b470f8cdbc5d︠
X
︡9c15e826-5850-4fd8-a379-04e5f6954a1f︡{"stdout": "[  1/4 -1/12 -1/12     0 -1/12     0     0     0]\n[-1/12  5/48  1/32 -1/24  1/32 -1/24     0     0]\n[-1/12  1/32  5/48 -1/24  1/32     0 -1/24     0]\n[    0 -1/24 -1/24  5/48     0  1/32  1/32 -1/12]\n[-1/12  1/32  1/32     0  5/48 -1/24 -1/24     0]\n[    0 -1/24     0  1/32 -1/24  5/48  1/32 -1/12]\n[    0     0 -1/24  1/32 -1/24  1/32  5/48 -1/12]\n[    0     0     0 -1/12     0 -1/12 -1/12   1/4]"}︡
︠c37ccedf-63a1-419d-a774-60801f635ef0︠
"""Unfortunately the ordering of the index set for the rows and columns differs from the paper (and this makes it difficult to see all of the symmetry.  Rows and columns are indexed by subsets of {0,1,2}.  The ordering is given by converting numbers into binary expansions and then viewing binary expansions as subsets.  So, 0,1,2,3,4,5,6,7 have binary expansions 000, 001, 010, 011, 100, 101, 110, 111 and these correspond to the subsets {}, {0}, {1}, {0,1}, {2}, {0,2}, {1,2}, {0,1,2}.  And this is how the index set for the rows and columns is ordered."""
︡b5e657dc-f517-4556-a675-9be9292ab694︡{"stdout": "'Unfortunately the ordering of the index set for the rows and columns differs from the paper (and this makes it difficult to see all of the symmetry.  Rows and columns are indexed by subsets of {0,1,2}.  The ordering is given by converting numbers into binary expansions and then viewing binary expansions as subsets.  So, 0,1,2,3,4,5,6,7 have binary expansions 000, 001, 010, 011, 100, 101, 110, 111 and these correspond to the subsets {}, {0}, {1}, {0,1}, {2}, {0,2}, {1,2}, {0,1,2}.  And this is how the index set for the rows and columns is ordered.'"}︡
︠c18054a3-b982-4ca3-a52f-0f80f5031c5e︠
"""Here is latex code for this matrix"""
︡49c5a4d3-006f-4648-9bde-a045ffc3cd24︡{"stdout": "'Here is latex code for this matrix'"}︡
︠d0a04f26-7565-4ab8-9ea0-d34439ae87a3︠
latex(X)
︡bf903840-e580-4e5d-a4ab-30e526a4ef39︡{"stdout": "\\left(\\begin{array}{rrrrrrrr}\n\\frac{1}{4} & -\\frac{1}{12} & -\\frac{1}{12} & 0 & -\\frac{1}{12} & 0 & 0 & 0 \\\\\n-\\frac{1}{12} & \\frac{5}{48} & \\frac{1}{32} & -\\frac{1}{24} & \\frac{1}{32} & -\\frac{1}{24} & 0 & 0 \\\\\n-\\frac{1}{12} & \\frac{1}{32} & \\frac{5}{48} & -\\frac{1}{24} & \\frac{1}{32} & 0 & -\\frac{1}{24} & 0 \\\\\n0 & -\\frac{1}{24} & -\\frac{1}{24} & \\frac{5}{48} & 0 & \\frac{1}{32} & \\frac{1}{32} & -\\frac{1}{12} \\\\\n-\\frac{1}{12} & \\frac{1}{32} & \\frac{1}{32} & 0 & \\frac{5}{48} & -\\frac{1}{24} & -\\frac{1}{24} & 0 \\\\\n0 & -\\frac{1}{24} & 0 & \\frac{1}{32} & -\\frac{1}{24} & \\frac{5}{48} & \\frac{1}{32} & -\\frac{1}{12} \\\\\n0 & 0 & -\\frac{1}{24} & \\frac{1}{32} & -\\frac{1}{24} & \\frac{1}{32} & \\frac{5}{48} & -\\frac{1}{12} \\\\\n0 & 0 & 0 & -\\frac{1}{12} & 0 & -\\frac{1}{12} & -\\frac{1}{12} & \\frac{1}{4}\n\\end{array}\\right)"}︡
︠2bbed968-1e6f-4f26-b756-3d0d3648e4be︠
"""Here I compute X's eigenvalues"""
︡85437371-9742-4411-8223-871e8e4a1367︡{"stdout": "\"Here I compute X's eigenvalues\""}︡
︠8c2bb1df-b421-4739-96f5-c157bc38aa29︠
X.eigenvalues()
︡23054725-b3b3-47ed-bffd-272e8f6d1f91︡{"stdout": "[1/3, 0, 11/96, 11/96, 1/32, 1/32, 0.10566243270259356?, 0.3943375672974065?]"}︡
︠4baf505e-e8ea-486d-957c-facd61e9a746︠
"""Since the eigenvalues are all non-negative I can conclude that the symmetrization of p, which is 1-(z_1+z_2+z_3+z_4)/4 is an Agler class denominator."""
︡34eff1fe-cb02-4337-8270-61989517b719︡{"stdout": "'Since the eigenvalues are all non-negative I can conclude that the symmetrization of p, which is 1-(z_1+z_2+z_3+z_4)/4 is an Agler class denominator.'"}︡
︠e12e1cde-de03-4505-b15b-9bda783ca620︠
"""Let's try this with the polynomial q(z) = 3-2z, viewed as having degree at most 5."""
︡dca365ad-baf9-4a05-a616-dbcb3ad0ee16︡{"stdout": "\"Let's try this with the polynomial q(z) = 3-2z, viewed as having degree at most 5.\""}︡
︠6acab31b-e01c-4d32-b89b-1e3a429e8a03︠
q = [3,-2,0,0,0,0]
︡be37a0e6-c2cf-492f-a13c-22726337f66a︡︡
︠25c93ba4-90c8-4522-9227-f20937785312︠
Y = matrix(Bmatrix(q))
︡b4147bb3-0bc0-4d24-95cc-19b97b17efec︡︡
︠37caedd9-569b-498f-91c4-76deb6f8d894︠
Y.eigenvalues()
︡11fd2731-43bf-45c3-8c9a-a3b9b25a9314︡{"stdout": "[41/150, 41/150, 131/300, 131/300, 131/300, 0.3939614930702669?, 2.056038506929733?, 0.1742419815397513?, 0.5890913517935820?, 0.1742419815397513?, 0.5890913517935820?, 0.1742419815397513?, 0.5890913517935820?, 0.0899809071728191?, 0.7255680527388265?, 2.067784373421688?]"}︡
︠e1c6bb95-e00b-413a-b094-34b29a39fda4︠
'''In this case Y is positive definite which would provide support to the conjecture that strict stability should make B strictly positive.'''
︡4f93722a-dd37-47b1-b177-093843aad06e︡{"stdout": "'In this case Y is positive definite which would provide support to the conjecture that strict stability should make B strictly positive.'"}︡
